/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific
   main-programmer: Benjamin Grauer
   co-programmer:
*/


#include "turret.h"

#include "weapon_manager.h"
#include "test_bullet.h"

#include "state.h"
#include "vector.h"
#include "list.h"
#include "animation3d.h"
#include "sound_engine.h"

#include "fast_factory.h"


using namespace std;


/**
 *  standard constructor

   creates a new weapon
*/
Turret::Turret (WeaponManager* weaponManager)
  : Weapon(weaponManager)
{
  this->setClassID(CL_TURRET, "Turret");

  this->model = (Model*)ResourceManager::getInstance()->load("models/turret1.obj", OBJ, RP_CAMPAIGN);

  Animation3D* animation1 = this->getAnimation(WS_ACTIVATING, this);
  Animation3D* animation2 = this->getAnimation(WS_DEACTIVATING, this);

  animation1->addKeyFrame(Vector(0, -.5, 0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_CONSTANT);
  animation1->addKeyFrame(Vector(0, 0, 0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_CONSTANT);
  animation2->addKeyFrame(Vector(0, 0, 0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_CONSTANT);
  animation2->addKeyFrame(Vector(0, -.5, 0), Quaternion(), 0.3, ANIM_LINEAR, ANIM_CONSTANT);

  animation1->setInfinity(ANIM_INF_CONSTANT);
  animation2->setInfinity(ANIM_INF_CONSTANT);

  this->setStateDuration(WS_SHOOTING, .4);
  this->setStateDuration(WS_RELOADING, 2);
  this->setStateDuration(WS_ACTIVATING, .4);
  this->setStateDuration(WS_DEACTIVATING, .4);

  this->setMaximumEnergy(1000, 10);
  this->increaseEnergy(100);
  //this->minCharge = 2;

  this->setActionSound(WA_SHOOT, "sound/shot1.wav");

  this->setProjectile(CL_TEST_BULLET);

  this->setEmissionPoint(1.684, 0.472, 0);
  //this->getProjectileFactory()->prepare(100);
}


/**
 *  standard deconstructor
*/
Turret::~Turret ()
{
  // model will be deleted from WorldEntity-destructor
}

void Turret::activate()
{
}

void Turret::deactivate()
{
}

void Turret::tick(float dt)
{
  Vector direction = this->getWeaponManager()->getFixedTarget()->getAbsCoor() - this->getAbsCoor();
  direction.normalize();
  Quaternion quat = Quaternion(direction, Vector(0,1,0)) * Quaternion ( -M_PI_2, Vector(0,1,0)) ;

  this->setAbsDir(quat);
}

/**
 *  fires the weapon

   this is called from the player.cc, when fire-button is been pushed
*/
void Turret::fire()
{
  Projectile* pj =  dynamic_cast<Projectile*>(this->getProjectileFactory()->resurrect());

  PNode* target = this->getWeaponManager()->getFixedTarget();

  if (target != NULL)
  {
    pj->setVelocity(this->getVelocity()+(target->getAbsCoor() - this->getAbsCoor())*.5);//this->getVelocity());
    pj->setAbsDir(this->getAbsDir());
  }
  else
    pj->setVelocity(target->getVelocity());


  pj->setAbsCoor(this->getEmissionPoint());
  pj->setAbsDir(this->getAbsDir());
  State::getWorldEntityList()->add(pj);
}


/**
 *  is called, when the weapon is destroyed

   this is in conjunction with the hit function, so when a weapon is able to get
   hit, it can also be destoryed.
*/
void Turret::destroy ()
{}

/**
 *  this will draw the weapon
*/
void Turret::draw ()
{
  this->getWeaponManager()->getFixedTarget()->debugDraw(10);

  /* draw gun body */
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  float matrix[4][4];
  glTranslatef (this->getAbsCoor ().x,
                this->getAbsCoor ().y,
                this->getAbsCoor ().z);
  this->getAbsDir ().matrix (matrix);
  glMultMatrixf((float*)matrix);

  this->model->draw();
  glPopMatrix();
}

